@@ -5,7 +5,7 @@ use crate::lower::{FieldType, Ir};
 use proc_macro2::{Ident, TokenStream};
 use quote::{format_ident, quote};
 use syn::punctuated::Punctuated;
-use syn::{Expr, ExprCall, GenericArgument, Generics, Index, Result, Type};
+use syn::{Expr, ExprCall, GenericArgument, Generics, Index, Result, Type, WhereClause};
 extern crate inflector;
 use inflector::Inflector;
 
@@ -34,7 +34,7 @@ pub fn codegen(ir: Ir) -> Result<TokenStream> {
     let (_, all_ty_generics, _) = all_generics.split_for_impl();
 
     let method_generics = &ir.method_generics;
-    let builder_init_generics = Generics::combine(vec![&ir.method_generics, &ir.generics]);
+    let builder_init_generics = Generics::combine(vec![&ir.generics, &ir.method_generics]);
     let target_generics_raw: Vec<GenericArgument> = builder_init_generics
         .to_generic_args()
         .args
@@ -46,7 +46,7 @@ pub fn codegen(ir: Ir) -> Result<TokenStream> {
     let constructor_return = &ir.return_type;
     let builder_method_name = &ir.builder_method_name;
     let builder_name = &ir.builder_name;
-    let builder_methods = builder_methods(&ir)?;
+    let builder_methods = builder_methods(&ir, builder_where_clause)?;
     let builder_state_type_initial = ir.builder_state_type_initial();
     let builder_state_initial = ir.builder_state_initial();
 
@@ -68,12 +68,12 @@ pub fn codegen(ir: Ir) -> Result<TokenStream> {
             {
                 #builder_name {
                     fields: (#(#builder_state_initial ,) *),
-                    phantom: core::default::Default::default()
+                    _phantom: core::default::Default::default()
                 }
             }
 
            pub struct __Required<T> {
-                phantom: std::marker::PhantomData<T>,
+                _phantom: std::marker::PhantomData<T>,
             }
             pub struct __Optional<T> {
                 lazy: Option<T>,
@@ -88,7 +88,7 @@ pub fn codegen(ir: Ir) -> Result<TokenStream> {
 
             fn __required<T>() -> __Required<T> {
                 __Required::<T> {
-                    phantom: core::default::Default::default(),
+                    _phantom: core::default::Default::default(),
                 }
             }
 
@@ -109,7 +109,7 @@ pub fn codegen(ir: Ir) -> Result<TokenStream> {
 
             pub struct #builder_name #all_ty_generics {
                 fields: __P,
-                phantom: (#(core::marker::PhantomData<#target_generics_raw>,) *)
+                _phantom: (#(core::marker::PhantomData<#target_generics_raw>,) *)
             }
 
             #(#builder_methods)*
@@ -123,8 +123,11 @@ pub fn codegen(ir: Ir) -> Result<TokenStream> {
     })
 }
 
-pub fn builder_methods(ir: &Ir) -> Result<Vec<TokenStream>> {
-    let builder_generics = Generics::combine(vec![&ir.method_generics, &ir.generics]);
+pub fn builder_methods(
+    ir: &Ir,
+    builder_where_clause: Option<&WhereClause>,
+) -> Result<Vec<TokenStream>> {
+    let builder_generics = Generics::combine(vec![&ir.generics, &ir.method_generics]);
     Ok(ir.builder_fields
         .iter()
         .enumerate()
@@ -145,7 +148,7 @@ pub fn builder_methods(ir: &Ir) -> Result<Vec<TokenStream>> {
 
             };
             let params_after = match f.field_type {
-                FieldType::Regular | FieldType::Optional =>
+                FieldType::Regular | FieldType::Option =>
                     builder_type_generics
                         .to_tuple_type()
                         .with_type(idx, f.ty.clone().wrap_in_generic(format_ident!("__Set"))),
@@ -168,20 +171,20 @@ pub fn builder_methods(ir: &Ir) -> Result<Vec<TokenStream>> {
             let generic_param = ty.generic_args();
 
             match f.field_type {
-                FieldType::Optional => {
+                FieldType::Option => {
                     let and_method_name = format_ident!("and_{}", f.name);
                     quote! {
                         impl #builder_type_generics #builder_name #before {
-                            pub fn #method_name (self, #field_name: #generic_param) -> #builder_name #after {
+                            pub fn #method_name (self, #field_name: #generic_param) -> #builder_name #after #builder_where_clause {
                                 #builder_name {
                                     fields: #new_state_option,
-                                    phantom: core::default::Default::default()
+                                    _phantom: core::default::Default::default()
                                 }
                             }
-                            pub fn #and_method_name (self, #field_name: #ty) -> #builder_name #after {
+                            pub fn #and_method_name (self, #field_name: #ty) -> #builder_name #after #builder_where_clause {
                                 #builder_name {
                                     fields: #new_state,
-                                    phantom: core::default::Default::default()
+                                    _phantom: core::default::Default::default()
                                 }
                             }
                         }
@@ -189,18 +192,29 @@ pub fn builder_methods(ir: &Ir) -> Result<Vec<TokenStream>> {
                 },
                 FieldType::Set => {
                     let (singular, plural) = single_plural_names(field_name);
-                    let field_collection_type = &f.collection_type;
+                    let mut field_collection_type = f.collection_type.clone();
+                    let mut into_generics = None;
+                    let mut into_call = None;
+                    if f.collection_into {
+                        let into_type = field_collection_type.replace(Type::parse("__T"));
+                        let _ = into_generics.insert(Some(quote! {
+                            <__T: Into<#into_type>>
+                        }));
+                        into_call = Some(quote!{
+                            .into()
+                        })
+                    }
                     let index = Index::from(idx);
                     quote! {
                         impl #builder_type_generics #builder_name #before {
 
-                            pub fn #plural (mut self, #field_name: #ty) -> #builder_name #before {
+                            pub fn #plural (mut self, #field_name: #ty) -> #builder_name #before #builder_where_clause {
                                 self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).extend(#field_name.into_iter());
                                 self
                             }
 
-                            pub fn #singular (mut self, value: #field_collection_type) -> #builder_name #before {
-                                self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).insert(value);
+                            pub fn #singular #into_generics(mut self, value: #field_collection_type) -> #builder_name #before #builder_where_clause{
+                                self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).insert(value #into_call);
                                 self
                             }
 
@@ -209,18 +223,30 @@ pub fn builder_methods(ir: &Ir) -> Result<Vec<TokenStream>> {
                 },
                 FieldType::Vec => {
                     let (singular, plural) = single_plural_names(field_name);
-                    let field_collection_type = &f.collection_type;
+                    let mut field_collection_type = f.collection_type.clone();
+                    let mut into_generics = None;
+                    let mut into_call = None;
+                    if f.collection_into {
+                        let into_type = field_collection_type.replace(Type::parse("__T"));
+                        let _ = into_generics.insert(Some(quote! {
+                            <__T: Into<#into_type>>
+                        }));
+                        into_call = Some(quote!{
+                            .into()
+                        })
+                    }
                     let index = Index::from(idx);
+
                     quote! {
                         impl #builder_type_generics #builder_name #before {
 
-                            pub fn #plural (mut self, #field_name: #ty) -> #builder_name #before {
+                            pub fn #plural (mut self, #field_name: #ty) -> #builder_name #before #builder_where_clause{
                                 self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).extend(#field_name.into_iter());
                                 self
                             }
 
-                            pub fn #singular (mut self, value: #field_collection_type) -> #builder_name #before {
-                                self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).push(value);
+                            pub fn #singular #into_generics(mut self, value: #field_collection_type) -> #builder_name #before #builder_where_clause{
+                                self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).push(value #into_call);
                                 self
                             }
 
@@ -229,19 +255,51 @@ pub fn builder_methods(ir: &Ir) -> Result<Vec<TokenStream>> {
                 },
                 FieldType::Map => {
                     let (singular, plural) = single_plural_names(field_name);
-                    let field_key_type = &f.key_type;
-                    let field_value_type = &f.value_type;
+                    let mut field_key_type = f.key_type.clone();
+                    let mut field_value_type = f.value_type.clone();
+                    let mut into_generics = Vec::new();
+                    let mut field_key_into_call = None;
+                    let mut field_value_into_call = None;
+                    if f.key_into {
+                        let into_type = field_key_type.replace(Type::parse("__K"));
+                        let _ = into_generics.push(quote! {
+                            __K: Into<#into_type>
+                        });
+                        field_key_into_call = Some(quote!{
+                            .into()
+                        })
+                    }
+                    if f.value_into {
+                        let into_type = field_value_type.replace(Type::parse("__V"));
+                        let _ = into_generics.push(quote! {
+                            __V: Into<#into_type>
+                        });
+                        field_value_into_call = Some(quote!{
+                            .into()
+                        })
+                    }
+
+                    let into_generics_final = if into_generics.is_empty() {
+                        None
+                    }
+                    else {
+                        Some(quote! {
+                            <#(#into_generics),*>
+                        })
+                    };
+
+
                     let index = Index::from(idx);
                     quote! {
                         impl #builder_type_generics #builder_name #before {
 
-                            pub fn #plural (mut self, #field_name: #ty) -> #builder_name #before {
+                            pub fn #plural (mut self, #field_name: #ty) -> #builder_name #before #builder_where_clause{
                                 self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).extend(#field_name.into_iter());
                                 self
                             }
 
-                            pub fn #singular (mut self, key: #field_key_type, value: #field_value_type) -> #builder_name #before {
-                                self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).insert(key, value);
+                            pub fn #singular #into_generics_final (mut self, key: #field_key_type, value: #field_value_type) -> #builder_name #before {
+                                self.fields.#index.lazy.get_or_insert_with(||core::default::Default::default()).insert(key #field_key_into_call, value #field_value_into_call);
                                 self
                             }
                         }
@@ -252,7 +310,7 @@ pub fn builder_methods(ir: &Ir) -> Result<Vec<TokenStream>> {
                         pub fn #method_name (self, #field_name: #ty) -> #builder_name #after {
                             #builder_name {
                                 fields: #new_state,
-                                phantom: core::default::Default::default()
+                                _phantom: core::default::Default::default()
                             }
                         }
                     }
@@ -391,4 +449,14 @@ mod tests {
     fn returns_self_test() {
         assert_codegen!(returns_self_test_case());
     }
+
+    #[test]
+    fn multiple_generics_test() {
+        assert_codegen!(multiple_generics_test_case());
+    }
+
+    #[test]
+    fn collection_generics_test2() {
+        assert_codegen!(collections_generics_test_case2());
+    }
 }
